//******************************************************************************
//  Demo Application02 for MSP430/LMP91050 Interface Code Library
//  Measure default common mode voltage using onboard ADC141S628 (w/ LMP91050 inputs 
//  shorted). Adjust DAC code and observe the output level shift.
//
//                MSP430F5528
//             -----------------
//         /|\|              XIN|-
//          | |                 |  
//          --|RST          XOUT|-
//            |                 |
//            |    P3.3/UCA0SIMO|--> MOSI
//            |    P3.4/UCA0SOMI|<-- MISO 
//            |     P2.7/UCA0CLK|--> CLK 
//            |             P2.5|--> CSB 
//            |             P2.6|--> MISO_EN 
//            |                 | 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   April 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91050.h"
#include "TI_LMP91050_register_settings.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_spi.h"

void timerA0_init(void);                                                       // initialize timerA0 to generate interrupt every sec
                                                                               // for temperature measurement
#define NUM_OF_RESULTS      8                                                  // Number of temp sensor samples to take
#define SCALE_FACTOR        3                                                  // For averaging converted samples
#define ADC14_RATIO         0.25                                               // 4096/16384(4096mV ref & 14bit converter & straight binary o/p)
#define INTERVAL_1S         16666                                              // 1sec timer interval
//******************************************************************************
void main(void)
{
  uint16_t reg_read_data1, reg_read_data2;
    
  WDTCTL = WDTPW+WDTHOLD;                                                      // Stop WDT
  
  TI_LMP91050_LED_PxOUT |= TI_LMP91050_LED_PIN;                                // Set LED ON
  TI_LMP91050_LED_PxDIR |= TI_LMP91050_LED_PIN;                                // Set pin direction is output
  
  TI_LMP91050_SPISetup();                                                      // Initilaize MSP430 SPI Block
  
  // Initialize Device and DAC Config Registers
  TI_LMP91050_SPIWriteReg(TI_LMP91050_DEV_CONFIG_REG, TI_LMP91050_DEV_CONFIG_REG_VALUE);          
  TI_LMP91050_SPIWriteReg(TI_LMP91050_DAC_CONFIG_REG, TI_LMP91050_DAC_CONFIG_REG_VALUE); 
  
  // write byte sequence to enter SDIO Mode
  TI_LMP91050_SPIWriteReg(TI_LMP91050_SDIO_MODE_ENABLE_REG, LMP91050_SDIO_SEQ_BYTE1);             
  TI_LMP91050_SPIWriteReg(TI_LMP91050_SDIO_MODE_ENABLE_REG, LMP91050_SDIO_SEQ_BYTE2);  

  reg_read_data1 = TI_LMP91050_SPIReadReg(TI_LMP91050_DEV_CONFIG_REG);         // Read back Device Config Register value   
  reg_read_data2 = TI_LMP91050_SPIReadReg(TI_LMP91050_DAC_CONFIG_REG);         // Read back Device Config Register value 
  
  // test if write/read values match
  if ((reg_read_data1 != TI_LMP91050_DEV_CONFIG_REG_VALUE) || 
      (reg_read_data2 != TI_LMP91050_DAC_CONFIG_REG_VALUE))
  {
   while (1)                                                                   // error: blink LED continuously
   {
     __delay_cycles(250000);
     TI_LMP91050_LED_PxOUT ^= TI_LMP91050_LED_PIN;        
   }      
  }
  
  TI_ADC14S628_SPISetup();                                                     // setup onboard ADS14S628 SPI interface
  timerA0_init();                                                              // initialize timerA0 for interrupt every 1sec                  
  __bis_SR_register(LPM0_bits + GIE);                                          // Enter LPM0, enable interrupts
  __no_operation();                                                            // For debugger 

} 
//------------------------------------------------------------------------------
//  void timerA0_init(void)
//
//  DESCRIPTION:
//  Initialize timer_A0 to generat 1sec interrupt.   
//  Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
void timerA0_init(void)
{
        
  TA0CCTL0 = CCIE;                                                             // TA2CCR0 interrupt enabled
  TA0CCR0 = INTERVAL_1S;                                                       // Initialize for 1sec interval
  TA0EX0 = TAIDEX_7;                                                           // Divide clock (selected below) further by 8	
  TA0CTL = TASSEL_2 + MC_1 + TACLR + ID_3;                                     // SMCLK, upmode, clear TAR, divide by 8
    
}
//------------------------------------------------------------------------------
// Timer A0 interrupt service routine
// Note: This function is for MSP430F5528 & might need modifications for other MSP430s
//------------------------------------------------------------------------------
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR (void)
{
  uint8_t i, reg_read_data;
  static uint16_t results[NUM_OF_RESULTS];                                     // To store ADC14 output
  static uint32_t sum_adc_data = 0;                                            // accumulate and avg adc results
  static float previous_vout = 1150;                                           // vout at start = cmn_mode
  volatile static float vout, step_change;                                     // voltage out 
  
  for(i=0; i<8;i++)
  {
    results[i] = TI_ADC14S628_SPIReadReg();                                    // read and store ADC14 output
    sum_adc_data += results[i];                                                // accumulate result
    __delay_cycles(4);                                                         // acquisition time                             
  }
  sum_adc_data >>= SCALE_FACTOR;                                               // Average the accumulated sum
  vout = ADC14_RATIO * sum_adc_data;                                           // convert to voltage
  step_change = vout - previous_vout;                                          // calculate effect of DAC code change
  sum_adc_data = 0;                                                            // Set Breakpoint here & see measured vout as well as
                                                                               // effect of DAC code change by -1  
  reg_read_data = TI_LMP91050_SPIReadReg(TI_LMP91050_DAC_CONFIG_REG);          // Read back current DAC code 
  reg_read_data -= 1;                                                          // Decrement DAC code by 1: o/p will increase
  if (!reg_read_data)                                                          // if dac code is equal to 0
    reg_read_data = 0x80;                                                      // set it back to 128
  TI_LMP91050_SPIWriteReg(TI_LMP91050_DAC_CONFIG_REG, reg_read_data);          // Write updated DAC code to DAC config
  previous_vout = vout;                                                        // Save vout
  
}  
//------------------------------------------------------------------------------
