//******************************************************************************
//  Description:  This file contains functions that allow the MSP430 device to
//  access the SPI interface of the LMP91050.  There are multiple
//  instances of each function; the one to be compiled is selected by the
//  system variable TI_LMP91050_SER_INTF, defined in "TI_LMP91050_hardware_board.h".
//
//  MSP430/LMP91050 Interface Code Library v1.0
// 
//
//   Vishy Natarajan
//   Texas Instruments Inc.
//   April 2012
//   Built with CCE Version: 4.2 and IAR Embedded Workbench Version:  5.3x
//******************************************************************************
// Change Log:
//******************************************************************************
// Version:  1.00
// Comments: Initial Release Version
//******************************************************************************
/*  Copyright 2011-2012 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user who
  downloaded the software, his/her employer (which must be your employer) and
  Texas Instruments Incorporated (the "License"). You may not use this Software
  unless you agree to abide by the terms of the License. The License limits your
  use, and you acknowledge, that the Software may not be modified, copied or
  distributed unless embedded on a Texas Instruments microcontroller which is 
  integrated into your product. Other than for the foregoing purpose, you may 
  not use, reproduce, copy, prepare derivative works of, modify, distribute, 
  perform, display or sell this Software and/or its documentation for any 
  purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL TEXAS
  INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL
  EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT
  LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL
  DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS,
  TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
  LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/

#include <stdint.h>
#include "TI_LMP91050.h"
#include "TI_MSP430.h"
#include "TI_MSP430_hardware_board.h"
#include "TI_MSP430_spi.h"


//******************************************************************************
// Support for 449x USART1
//******************************************************************************
#if TI_LMP91050_SER_INTF == TI_LMP91050_SER_INTF_USART1

//------------------------------------------------------------------------------
//  void TI_LMP91050_SPISetup(void)
//
//  DESCRIPTION:
//  Configures the assigned interface to function as a SPI port and
//  initializes it.
//------------------------------------------------------------------------------
void TI_LMP91050_SPISetup(void)
{
  TI_LMP91050_CSn_PxOUT |= TI_LMP91050_CSn_PIN;
  TI_LMP91050_CSn_PxDIR |= TI_LMP91050_CSn_PIN;                                // /CS disable
  
  TI_LMP91050_MOSIen_PxOUT |= TI_LMP91050_MOSIen_PIN;
  TI_LMP91050_MOSIen_PxDIR |= TI_LMP91050_MOSIen_PIN;                          // /MOSIen disable  

  ME2 |= USPIE1;                                                               // Enable USART1 SPI mode
  UCTL1 = SWRST;                                                               // Disable USART state machine
  UCTL1 |= CHAR + SYNC + MM;                                                   // 8-bit SPI Master **SWRST**
  UTCTL1 |= CKPL + SSEL1 + SSEL0 + STC;                                        // SMCLK, 3-pin mode
  UBR01 = 0x02;                                                                // UCLK/2
  UBR11 = 0x00;                                                                // 0
  UMCTL1 = 0x00;                                                               // No modulation
  TI_LMP91050_SPI_USART1_PxSEL |= TI_LMP91050_SPI_USART1_SIMO
                          | TI_LMP91050_SPI_USART1_SOMI
                          | TI_LMP91050_SPI_USART1_UCLK;
                                                                               // SPI option select
  TI_LMP91050_SPI_USART1_PxDIR |= TI_LMP91050_SPI_USART1_SIMO 
                                  + TI_LMP91050_SPI_USART1_UCLK;               // SPI TX out direction
  UCTL1 &= ~SWRST;                                                             // Initialize USART state machine
}

//------------------------------------------------------------------------------
//  void TI_LMP91050_SPIWriteReg(uint8_t addr, uint8_t value)
//
//  DESCRIPTION:
//  Writes "value" to a configuration register at address "addr".
//------------------------------------------------------------------------------
void TI_LMP91050_SPIWriteReg(uint8_t addr, uint8_t value)
{
  
  TI_LMP91050_CSn_PxOUT &= ~TI_LMP91050_CSn_PIN;                               // /CS enable  
  TI_LMP91050_MOSIen_PxOUT &= ~TI_LMP91050_MOSIen_PIN;                         // /MOSIen enable   
  
  while (!(IFG2 & UTXIFG1));                                                   // Wait for TXBUF ready
  IFG2 &= ~URXIFG1;                                                            // clear flag
  U1TXBUF = (~LMP91050_RW_BIT) & addr;                                         // Send write_bit + address
  while (!(IFG2 & URXIFG1));                                                   // wait for tx to finish 
  
  while (!(IFG2 & UTXIFG1));                                                   // Wait for TXBUF ready
  IFG2 &= ~URXIFG1;                                                            // clear flag
  U1TXBUF = value;                                                             // Send data value
  while (!(IFG2 & URXIFG1));                                                   // wait for tx to finish  

  TI_LMP91050_MOSIen_PxOUT |= TI_LMP91050_MOSIen_PIN;                          // /MOSIen disable  
  TI_LMP91050_CSn_PxOUT |= TI_LMP91050_CSn_PIN;                                // /CS disable   
  
}
//------------------------------------------------------------------------------
//  uint8_t TI_LMP91050_SPIReadReg(uint8_t addr)
//
//  DESCRIPTION:
//  Reads a configuration register at address "addr" and returns the
//  value read.
//------------------------------------------------------------------------------
uint8_t TI_LMP91050_SPIReadReg(uint8_t addr)
{
  uint8_t x;
  
  TI_LMP91050_CSn_PxOUT &= ~TI_LMP91050_CSn_PIN;                               // /CS enable  
  TI_LMP91050_MOSIen_PxOUT &= ~TI_LMP91050_MOSIen_PIN;                         // /MOSIen enable   
   
  while (!(IFG2 & UTXIFG1));                                                   // Wait for TXBUF ready
  IFG2 &= ~URXIFG1;                                                            // clear flag
  U1TXBUF = LMP91050_RW_BIT | addr;                                            // Send read_bit + address
  while (!(IFG2 & URXIFG1));                                                   // wait for tx to finish    
  
  TI_LMP91050_MOSIen_PxOUT |= TI_LMP91050_MOSIen_PIN;                          // /MOSIen disable for read
  
  while (!(IFG2 & UTXIFG1));                                                   // Wait for TXBUF ready
  IFG2 &= ~URXIFG1;                                                            // clear flag
  U1TXBUF = 0;                                                                 // Dummy write to read data
  while (!(IFG2 & URXIFG1));                                                   // wait for tx to finish  
  
  x = U1RXBUF;                                                                 // Read data   
  TI_LMP91050_CSn_PxOUT |= TI_LMP91050_CSn_PIN;                                // /CS disable 
  
  return x;
  
}
//------------------------------------------------------------------------------
#endif
